{
*****************************************************************************
*  Name    : USB Up Counter                                                 *
*  Author  : Hiroyuki Iizuka                                                *
*  Date    : 2010/03/24                                                     *
*  Version : 1.0                                                            *
*  Notes   :                                                                *
*          :                                                                *
*****************************************************************************
}
Program USBDounCounter

'---- configuration ---------------------------------------------------------
Device = 18F4550
Clock  = 48

Config
   USBDIV  = 1, 
   PLLDIV  = 5,
   CPUDIV  = OSC1_PLL2,   
   FOSC    = HSPLL_HS,
   MCLRE   = OFF

'---- EEPROM initialize -----------------------------------------------------
EEPROM = (0 As LongWord)

'---- include file ----------------------------------------------------------
Include "Convert.bas"
Include "EEPROM.bas"

'---- interrupt -------------------------------------------------------------
#option org_reset     = $2000
#option vector_isr_hi = $2008
#option vector_isr_lo = $2018

Const ipLow  = 1
Const ipHigh = 2

Const High_priority = 1
Const Low_priority  = 0

Dim Interrupt_HIGH_enable As INTCON.booleans(7)
Dim Interrupt_LOW_enable  As INTCON.booleans(6)
Dim Interrupt_priority    As RCON.booleans(7)

'------------ ROTARY ENCODER ------------------------------------------------
Dim ENCODER_A As PORTA.0
Dim ENCODER_B As PORTA.1

Const CW             =  1
Const CCW            = -1
Const ENCODER_STOP   =  0

Const ROTARY_LOCK    = 1
Const ROTARY_UNLOCK  = 0

Dim Rotary_Enc_A   As Byte
Dim Rotary_Enc_B   As Byte
Dim Rotary_A_Lock  As Boolean
Dim Rotary_B_Lock  As Boolean
Dim Rotary_Stat    As ShortInt
Dim turn           As ShortInt

'------------ Push Switch ---------------------------------------------------
Dim SW1 As PORTB.4
Dim SW2 As PORTA.3

Const SW_ON          = 0
Const SW_OFF         = 1

Const SW_ENABLE      = 0
Const SW_DISABLE     = 1

Dim SW1_status As Byte
Dim SW2_status As Byte

'------------ Mechanical RELAY ----------------------
Dim RELAY As PORTA.5

Const RELAY_ON       = 0
Const RELAY_OFF      = 1

'------------ 3mm LED -------------------------------------------------------
Dim LED1 As PORTB.5
Dim LED2 As PORTB.3
Dim LED3 As PORTA.2

'---- 7segment LED const definitions ----------------------------------------
Const Digit_N  = $00   ' Blank
Const Digit_0  = $3F
Const Digit_1  = $06
Const Digit_2  = $5B
Const Digit_3  = $4F
Const Digit_4  = $66
Const Digit_5  = $6D
Const Digit_6  = $7D
Const Digit_7  = $07
Const Digit_8  = $7F
Const Digit_9  = $6F
Const Digit_DP = $80

Const LED_ON        = 1
Const LED_OFF       = 0

Dim LED_DEV6     As PORTC.0
Dim LED_DEV5     As PORTC.1
Dim LED_DEV4     As PORTC.2
Dim LED_DEV3     As PORTE.0
Dim LED_DEV2     As PORTE.1
Dim LED_DEV1     As PORTE.2

Dim LED_SEG_PORT As PORTD
Dim LED_SEG_A    As PORTD.0
Dim LED_SEG_B    As PORTD.1
Dim LED_SEG_C    As PORTD.2
Dim LED_SEG_D    As PORTD.3
Dim LED_SEG_E    As PORTD.4
Dim LED_SEG_F    As PORTD.5
Dim LED_SEG_G    As PORTD.6
Dim LED_SEG_DP   As PORTD.7

Dim led_val1       As Byte
Dim led_val2       As Byte
Dim led_val3       As Byte
Dim led_val4       As Byte
Dim led_val5       As Byte
Dim led_val6       As Byte
Dim led_pos        As Byte

Dim set_val(6)  As Byte

'---- HALL sensor -----------------------------------------------------------
Dim PHOTO_REF_PORT As PORTB.0

'-------- INT0 --------------------------------------------------------------
Dim PortB0_changed         As INTCON.booleans(1)
Dim PortB0_int_enable      As INTCON.booleans(4)
Dim PortB0_int_rising_edge As INTCON2.booleans(6)

'-------- TMR0 --------------------------------------------------------------
Dim Timer0_overflowd             As INTCON.booleans(2)
Dim Timer0_interrupt_enable      As INTCON.booleans(5)
Dim Timer0_priority              As INTCON2.bits(2)
Dim Timer0_enable                As T0CON.booleans(7)
Dim Timer0_mode_8bit             As T0CON.booleans(6)
Dim Timer0_use_external_clock    As T0CON.booleans(5)
Dim Timer0_prescale_NOT_assign   As T0CON.booleans(3)
Dim Timer0_prescale_bit2         As T0CON.bits(2)
Dim Timer0_prescale_bit1         As T0CON.bits(1)
Dim Timer0_prescale_bit0         As T0CON.bits(0)

'-------- TMR1 --------------------------------------------------------------
Dim Timer1_overflowd             As PIR1.booleans(0) 
// Dim Timer1_peripheral_int_enable as INTCON.booleans(6)
Dim Timer1_interrupt_enable      As PIE1.booleans(0) 
Dim Timer1_prescale_bit0         As T1CON.bits(4)
Dim Timer1_prescale_bit1         As T1CON.bits(5)
Dim Timer1_use_external_clock    As T1CON.booleans(1)
Dim Timer1_enable                As T1CON.booleans(0)
Dim Timer1_priority              As IPR1.bits(0)

'-------- A/D converter -----------------------------------------------------
Dim ADconfig3 As ADCON1.3
Dim ADconfig2 As ADCON1.2
Dim ADconfig1 As ADCON1.1
Dim ADconfig0 As ADCON1.0

'-------- comparator --------------------------------------------------------
Dim Comparator_mode2 As CMCON.bits(2)
Dim Comparator_mode1 As CMCON.bits(1)
Dim Comparator_mode0 As CMCON.bits(0)

'-------- PORTB -------------------------------------------------------------
Dim PortB_pullups_disable As INTCON2.booleans(7)

'---- application [Down Counter] --------------------------------------------
Dim count          As LongWord
Dim target_count   As LongWord
Dim count_status   As Byte

Const READY = 0
Const SET   = 1


'============================================================================
' interrupt priority HIGH
'============================================================================
Interrupt OnChangePORTB(ipHigh)
  '---- signal was detected
  If PortB0_changed Then
    count = count + 1
    PortB0_changed = false
  End If
End Interrupt


'============================================================================
' interrupt priority LOW
'============================================================================
Interrupt OnTimer(ipLow)

  '--- TMR0 overflow (7SEG_LED storobe)   2 msec
  If Timer0_overflowd Then
    TMR0L = 69
    Select led_pos
      Case 1
        LED_DEV6 = LED_OFF
        LED_SEG_PORT = led_val1
        LED_DEV1 = LED_ON
        led_pos = led_pos + 1
      Case 2
        LED_DEV1 = LED_OFF
        LED_SEG_PORT = led_val2
        LED_DEV2 = LED_ON
        led_pos = led_pos + 1
      Case 3
        LED_DEV2 = LED_OFF
        LED_SEG_PORT = led_val3
        LED_DEV3 = LED_ON
        led_pos = led_pos + 1
      Case 4
        LED_DEV3 = LED_OFF
        LED_SEG_PORT = led_val4
        LED_DEV4 = LED_ON
        led_pos = led_pos + 1
      Case 5
        LED_DEV4 = LED_OFF
        LED_SEG_PORT = led_val5
        LED_DEV5 = LED_ON
        led_pos = led_pos + 1
      Case 6
        LED_DEV5 = LED_OFF
        LED_SEG_PORT = led_val6
        LED_DEV6 = LED_ON
        led_pos = 1
    EndSelect

    Timer0_overflowd = false   ' Clear interrupt flag
  End If
End Interrupt


'============================================================================
' decode ascii-char -> 7segment-LED
'============================================================================
Function led7seg_decode_ascii( ascii_char As Char ) As Byte
  Select ascii_char
    Case " "  result = Digit_N
    Case "0"  result = Digit_0
    Case "1"  result = Digit_1
    Case "2"  result = Digit_2
    Case "3"  result = Digit_3
    Case "4"  result = Digit_4
    Case "5"  result = Digit_5
    Case "6"  result = Digit_6
    Case "7"  result = Digit_7
    Case "8"  result = Digit_8
    Case "9"  result = Digit_9
    Else result = Digit_N
  EndSelect
End Function

'============================================================================
' decode number -> 7segment-LED
'============================================================================
Function led7seg_decode_number( num As Byte ) As Byte
  Select num
    Case 0  result = Digit_0
    Case 1  result = Digit_1
    Case 2  result = Digit_2
    Case 3  result = Digit_3
    Case 4  result = Digit_4
    Case 5  result = Digit_5
    Case 6  result = Digit_6
    Case 7  result = Digit_7
    Case 8  result = Digit_8
    Case 9  result = Digit_9
  EndSelect
End Function


'============================================================================
' Rotary encoder
'
' RETURN
'    CW / CCW / OTHER
'============================================================================
Function rotary_encoder() As ShortInt
  Dim temp_A As Byte
  Dim temp_B As Byte

  temp_A = ENCODER_A
  temp_B = ENCODER_B

  result = ENCODER_STOP

  If (Rotary_Enc_A = 1) And (temp_A = 0) And (temp_B = 1) Then
    Rotary_Stat   = CW
    Rotary_A_Lock = True
  End If
  If Rotary_A_Lock = True Then
    If (temp_A = 0) And (temp_B = 0) Then
      result = Rotary_Stat
      Rotary_A_Lock = False
    End If
  End If

  If (Rotary_Enc_B = 1) And (temp_B = 0) And (temp_A = 1) Then
    Rotary_Stat   = CCW
    Rotary_B_Lock = True
  End If
  If Rotary_B_Lock = True Then
    If (temp_A = 0) And (temp_B = 0) Then
      result = Rotary_Stat
      Rotary_B_Lock = False
    End If
  End If

  Rotary_Enc_A = temp_A
  Rotary_Enc_B = temp_B
End Function


'============================================================================
'  number -> led_val1 to led_val6
'============================================================================
Sub decode_number( num As LongWord )
  Dim led_txt  As String(6)
  
  led_txt = DecToStr( num, 6, " " )
  led_val1 = led7seg_decode_ascii( led_txt(5) )
  led_val2 = led7seg_decode_ascii( led_txt(4) )
  led_val3 = led7seg_decode_ascii( led_txt(3) )
  led_val4 = led7seg_decode_ascii( led_txt(2) )
  led_val5 = led7seg_decode_ascii( led_txt(1) )
  led_val6 = led7seg_decode_ascii( led_txt(0) )
End Sub


'============================================================================
'  number -> led_val1 to led_val6
'============================================================================
Sub decode_number_setmode( dp_pos As Byte )
  Dim tmp(6) As Byte
  Dim i As Byte
  
  For i = 0 To 5
    tmp(i) = led7seg_decode_number( set_val(i) )
  Next
  
  tmp(dp_pos) = tmp(dp_pos) Or Digit_DP
  
  led_val1 = tmp(0)
  led_val2 = tmp(1)
  led_val3 = tmp(2)
  led_val4 = tmp(3)
  led_val5 = tmp(4)
  led_val6 = tmp(5)
End Sub


'****************************************************************************
' Program main
'****************************************************************************
Dim dp_pos    As Byte
Dim tmp_count As LongWord
Dim i         As Byte
Dim multiple  As LongWord

PortB_pullups_disable  = true

PortB0_int_enable      = true
PortB0_int_rising_edge = true

ADconfig0 = 1  ' PortA Digital I/O
ADconfig1 = 1
ADconfig2 = 1 
ADconfig3 = 1

Comparator_mode2 = 1 ' compatator off
Comparator_mode1 = 1 ' compatator off
Comparator_mode0 = 1 ' compatator off
 

Input(PHOTO_REF_PORT)

Input(ENCODER_A)
Input(ENCODER_B)

Input(SW1)
Input(SW2)

Output(RELAY)

Output(LED1)
Output(LED2)
Output(LED3)

Output(LED_DEV6)
Output(LED_DEV5)
Output(LED_DEV4)
Output(LED_DEV3)
Output(LED_DEV2)
Output(LED_DEV1)
   
Output(LED_SEG_A)
Output(LED_SEG_B)
Output(LED_SEG_C)
Output(LED_SEG_D)
Output(LED_SEG_E)
Output(LED_SEG_F)
Output(LED_SEG_G)
Output(LED_SEG_DP)

LED_DEV6 = LED_OFF
LED_DEV5 = LED_OFF
LED_DEV4 = LED_OFF
LED_DEV3 = LED_OFF
LED_DEV2 = LED_OFF
LED_DEV1 = LED_OFF

led_val1 = Digit_8
led_val2 = Digit_8
led_val3 = Digit_8
led_val4 = Digit_8
led_val5 = Digit_8
led_val6 = Digit_8
led_pos  = 1  


'---- TMR0(8bitMODE) initialize --------------
Timer0_prescale_bit2 = 1   ' PreScaler 1:128
Timer0_prescale_bit1 = 1   ' PreScaler 1:128
Timer0_prescale_bit0 = 0   ' PreScaler 1:128
Timer0_prescale_NOT_assign = false
Timer0_mode_8bit           = true
Timer0_interrupt_enable    = true
Timer0_use_external_clock  = false
Timer0_enable              = true
Timer0_priority            = Low_priority

'---- interrpt -------------------------------
Enable(OnChangePORTB)
Enable(OnTimer)
Interrupt_priority    = true
Interrupt_HIGH_enable = true
Interrupt_LOW_enable  = true
 
Rotary_Enc_A = ENCODER_A
Rotary_Enc_B = ENCODER_B
 
LED1 = LED_OFF
LED2 = LED_OFF

 
RELAY = RELAY_OFF 
LED3 = LED_OFF 

DelayMS(1000)

target_count = ReadLongWord(0)
count = 0
  
count_status = READY

SW1_status = SW_ENABLE 
SW2_status = SW_ENABLE 

dp_pos = 0
While( true )
  '---- COUNT MODE <-> SET MODE ----------
  If SW2 = SW_ON Then
    If SW2_status = SW_ENABLE Then
      '---- normal mode -> set mode ------
      If count_status = READY Then
        count_status = SET
        
        tmp_count = ReadLongWord(0)
        For i = 0 To 5
          set_val(i) = tmp_count Mod 10
          tmp_count = tmp_count / 10
        Next
 
        LED1 = LED_ON
      '---- set mode -> normal mode ------
      Else
        count_status = READY
        LED1 = LED_OFF
        
        tmp_count = 0
        multiple = 1
        For i = 0 To 5
          tmp_count = tmp_count + (LongWord(set_val(i)) * multiple)
          multiple = multiple * 10 
        Next
        target_count = tmp_count
        count        = 0
        
        WriteLongWord(0, tmp_count)
        RELAY = RELAY_OFF
        LED3 = LED_OFF
      End If
      SW2_status = SW_DISABLE
    End If
  End If

  If SW2_status = SW_DISABLE Then  
    If SW2 = SW_OFF Then
      SW2_status = SW_ENABLE
    End If
  End If
    
  '---- COUNT MODE -----------------------
  If count_status = READY Then
    '---- reset ----------
    If SW1 = SW_ON Then
      If SW1_status = SW_ENABLE Then
        count = 0
        RELAY = RELAY_OFF
        LED3 = LED_OFF  
      End If
      SW1_status = SW_DISABLE
    End If

    If SW1_status = SW_DISABLE Then  
      If SW1 = SW_OFF Then
        SW1_status = SW_ENABLE
      End If
    End If
  
    If count = target_count Then
      RELAY = RELAY_ON
      LED3 = LED_ON
    End If
    decode_number( count )
  End If
  
  '---- SET MODE -------------------------
  If count_status = SET Then  
    
    '---- move decimal place ----------
    If SW1 = SW_ON Then
      If SW1_status = SW_ENABLE Then
        If dp_pos = 5 Then
          dp_pos = 0
        Else
          dp_pos = dp_pos + 1
        End If
      End If
      SW1_status = SW_DISABLE
    End If

    If SW1_status = SW_DISABLE Then  
      If SW1 = SW_OFF Then
        SW1_status = SW_ENABLE
      End If
    End If
    
    '---- rotary encoder --------------
    turn = rotary_encoder()
    Select turn
      Case CW 
        If set_val(dp_pos) < 9 Then
          set_val(dp_pos) = set_val(dp_pos) + 1
        End If
      Case CCW
        If set_val(dp_pos) > 0 Then
          set_val(dp_pos) = set_val(dp_pos) - 1
        End If
    EndSelect
             
    decode_number_setmode( dp_pos )         
  End If
Wend


'--------- E N D  O F  P R O G R A M ----------------------------------------
